function [b_idx_xC, b_idx_xWC, b_x_res, b_C, min_error, b_exitflag] = mL1nc_1(run_ind, n_com)

    % this function tries to get the smallest SAE by picking n_com components from
    % 2*n_comp ones. Half is from LibC and the other half is from LibWC
    
    addpath('C:\Program Files\IBM\ILOG\CPLEX_Studio1262\cplex\matlab\x64_win64');
    LibC = load(['LibCtest' int2str(run_ind) '.txt'])';
    LibWC = load(['LibWCtest' int2str(run_ind) '.txt'])';
    tmp = load(['Xtest' int2str(run_ind) '.txt']);
    
    min_error = Inf;
    
    for kk = 1:100
    
        Y = tmp(:,end);
        no_selected_sub_trees = 10;
        if size(LibC,2) > no_selected_sub_trees
            lib_idC = randperm(size(LibC,2), no_selected_sub_trees); % randomly pick no_selected_sub_trees trees from the LibC
        else
            lib_idC = 1:size(LibC,2);
        end
        if size(LibWC,2) > no_selected_sub_trees
            lib_idWC = randperm(size(LibWC,2), no_selected_sub_trees); % randomly pick no_selected_sub_trees trees from the LibWC
        else
            lib_idWC = 1:size(LibWC,2);
        end
        LibO = [LibC(:,lib_idC) LibWC(:,lib_idWC)];   % orginial Lib
        LibN = LibO;            % normalize Lib

        % normalize 
        minLib = repmat(min(LibN),[size(LibN,1),1]);
        maxLib = repmat(max(LibN),[size(LibN,1),1]);
        LibN = (LibN-minLib)./(maxLib-minLib);
        [~, col] = find(isnan(LibN));
        LibN(:,col) = 0;     % remove divide by zero

        minY = min(Y);
        maxY = max(Y);
        Y = (Y - minY)./(maxY-minY);

        % remove the correllations
        RHO = corr(LibN);       % Pearson correlation
        id = 1:size(LibN,2);
        for i = 1:size(LibN,2)-1;
            if (id(i) == -1)
                continue;
            end
            data = abs(RHO(i,i+1:end));
            [~, c] = find(data == 1);
            id(i+c) = -1;
        end
        idx_nc = find(id' ~= -1);   % ids of columns whose correllations are not 1 or -1
        LibNCorr = LibN(:,idx_nc);      % non-correlated Lib

        %
        % LP: minimize L1 subject to N components
        %

        % add column of 1 into LibC to deal with constant difference
        LibNCorr = [LibNCorr ones(size(LibNCorr,1),1)];
        LibNCorr = [LibNCorr -LibNCorr];    % add negative part

        % inequality constrains
        Aieq = [LibNCorr zeros(size(LibNCorr))];    % add variables to know which columns are used
        z = diag(-ones(size(LibNCorr,1),1));    % z is the eps to be minimized
        Aieq = [Aieq z; -Aieq z];
        bieq = [Y; -Y];

        % constrains to set a variable to be 1 if a columns is used, 0 otherwise
        % 1*xi + (-1)*ti <= 0, ti = {0,1}
        Aieq(end+1:end+size(LibNCorr,2),:) = [diag(ones(size(LibNCorr,2),1)) -1*diag(ones(size(LibNCorr,2),1)) zeros(size(LibNCorr,2),size(LibNCorr,1))];
        bieq(end+1:end+size(LibNCorr,2)) = zeros(size(LibNCorr,2),1);

        % constrain to limit the number of components to n_com
        % t1 + t2 + ... + tn <= n_com, ti = {0,1}
        Aieq(end+1,:) = [zeros(1, size(LibNCorr,2)) ones(1,size(LibNCorr,2)) zeros(1, size(LibNCorr,1))];
        bieq(end+1) = n_com;

        % objective function: minimize sum of zi, zi > 0
        f = [zeros(1,2*size(LibNCorr,2)) ones(1,size(LibNCorr,1))];

        % all the variables must >= 0 (because there is negative part of libC
        % to deal with <0 case)
        % upper bound is set to 1 as LibC has been normalized -> speed up the
        % LP. z can be anything from 0 -> inf, but can't be > 1 in reality
        lb = zeros(1,size(Aieq,2));
        ub = [1*ones(1,size(LibNCorr,2)) ones(1,size(LibNCorr,2)) ones(1,size(LibNCorr,1))];

        % flag to use mix integer LP 
        ctype = [repmat('C', [1 size(LibNCorr,2)]) repmat('B', [1 size(LibNCorr,2)]) repmat('C', [1 size(LibNCorr,1)])];

        % call LP and measure the runtime
%         tic;
%         [x,fval,exitflag] = cplexmilp(f,Aieq,bieq,[],[],[],[],[],lb,ub,ctype);
%         toc
        
        % use only 4 cores 
        cplex=Cplex();
        cplex.Model.name = 'test_mip';
        cplex.Model.lb = lb;
        cplex.Model.ub = ub;
        cplex.Model.obj = f;
        cplex.Model.A = Aieq;
        cplex.Model.rhs = bieq;
        cplex.Model.lhs = -inf*ones(size(bieq));
        cplex.Model.sense = 'minimize';
        cplex.Model.ctype = ctype;
        cplex.Param.threads.Cur = 4;
        
%         tic;
        cplex.solve()
%         toc

        x = cplex.Solution.pool.solution.x;
        exitflag = 0;
        if cplex.Solution.status == 101
            exitflag = 101; % status is an int -> must convert to double
        end
        fval = cplex.Solution.bestobjval;

        delete('*.log');
        if exitflag ~= 101 
            continue;
        end

%         error = sum(x(end-numel(Y)+1:end))

        % find x > 0
        x = reshape(x(1:end-size(LibNCorr,1)), [size(LibNCorr,2) 2]);
        x = x(:,1).*x(:,2);
        idx_x = find(x > 1.e-5);
        x_resN = x(idx_x);  % x result of normalized lib

        % back to the index of positive/original LibC
        % negative part is indicated by minus sign
        org_LibC_size = size(LibNCorr,2)/2;
        idx_x(idx_x > org_LibC_size) = -(idx_x(idx_x > org_LibC_size) - org_LibC_size);

        % back to the index before reducing corellated columns
        idx_nc(end+1) = size(LibN,2)+1; % column of 1 is added after normalization
                                        % its index is out of the range for idx_nc
        idx1 = idx_nc(abs(idx_x));
        idx1(idx_x < 0) = -idx1(idx_x < 0);
        idx_x = idx1;

        %
        % scale x_resN back to before normalization
        %
        LibO = [LibO ones(size(LibO,1),1)]; % add column of all ones to LibO
        Ax = LibO(:,abs(idx_x)); 
        minAx = min(Ax);
        maxAx = max(Ax);

        col1_idx = find(abs(idx_x) == size(LibO,2)); % search for the idx of column of 1
        if numel(col1_idx) > 0
            minAx(col1_idx) = 0;    % avoid divide by 0
        end

        % scale back x_resN
        x_res = x_resN'./(maxAx - minAx)*(maxY - minY);
        x_res = x_res';

        x_res(idx_x < 0) = -x_res(idx_x < 0);
        idx_x = abs(idx_x);
        C = -minAx * x_res + minY;

    %     checking result
        final = [Ax*x_res + C  tmp(:,end)];
        error = sum(abs(final(:,1)-final(:,2)));
%         sum(abs(final(:,1)-final(:,2)))/(maxY - minY)

        if (error < min_error)
            % convert idx_x to the original library index
            lib_id = [lib_idC lib_idWC size(LibWC,2)+1];  % in case the column of 1 is picked
            b_idx_xC = lib_id(idx_x(idx_x <= no_selected_sub_trees));
            b_idx_xWC = lib_id(idx_x(idx_x > no_selected_sub_trees)); % include columns of 1

            x_resC = x_res(idx_x <= no_selected_sub_trees);
            x_resWC = x_res(idx_x > no_selected_sub_trees);
            b_x_res = [x_resC; x_resWC];
            b_C = C;
            b_exitflag = exitflag;

%             LibWC = [LibWC ones(size(LibWC,1),1)];
%             Ax = [LibC(:,b_idx_xC) LibWC(:,b_idx_xWC)]; 
%             final = [Ax*b_x_res + b_C  tmp(:,end)]
%             sum(abs(final(:,1)-final(:,2)))
%             sum(abs(final(:,1)-final(:,2)))/(maxY - minY) 
%             plot(final(:,1), final(:,2), 'bo')
%             close;
            min_error = error;
        end   
    end
end










